#include <avr/io.h>
#include <stdint.h>
#include "LCD/lcd.h"
#include <stdlib.h>
#include <avr/interrupt.h>


//globale Variablendeklarationen
struct {								//mytime: Zeit der internen Uhr
	uint16_t milliseconds;
	uint8_t seconds;
	uint8_t minutes;
	uint8_t hours;
} mytime;

char string [3];							//Zhlvariable

//Funktionen
void init()
{
	//LCD-Display initialisieren
	lcd_init(LCD_DISP_ON_CURSOR);
	lcd_clrscr();
	lcd_gotoxy( 0,0 );
	lcd_puts("Time:");					//schreibt "Time:" in die erste Zeile

	//Timer1 initialisieren
	//zhlt 1 Sekunde (1MHz/64/15625 = 1s)
	TCCR1B = (1<<WGM12);				//CTC Modus aktivieren (Vergleicht Datenregister TCNT1H/TCNT1L mit Vergleichswert OCR1H/OCR1L)
	TCCR1B |= (1<<CS11) | (1<<CS10);	//Prescaler 64 (Timer 1 zhlt mit CPU-Takt/64)
	OCR1A = 15625-1;						//Datenregister
	TIMSK |= (1<<OCIE1A);				//Compare Interrupt erlauben
	
	//globale Interrupts aktivieren
	sei();
}

void incrementTimeS()					//zhlt die interne Zeit um 1 Sekunde hoch
{
	mytime.seconds ++;
	if(mytime.seconds == 60)
	{
		mytime.seconds = 0;
		mytime.minutes ++;

		if(mytime.minutes == 60)
		{
			mytime.minutes = 0;
			mytime.hours ++;

			if(mytime.hours == 24)
			{
				mytime.hours = 0;
			}
		}
	}
}

void lcd_printTime()					//gibt die interne Zeit auf dem LCD-Display aus
{
	lcd_gotoxy( 0,1 );
	if(mytime.hours < 10)				//wenn ein Wert nicht 2-Stellig wird trotzdem eine 0 davor geschrieben
		lcd_puts("0");
	itoa(mytime.hours, string, 10);
	lcd_puts(string);
	if(mytime.minutes < 10)
		lcd_puts(":0");
	else
		lcd_puts(":");
	itoa(mytime.minutes, string, 10);
	lcd_puts(string);
	if(mytime.seconds < 10)
		lcd_puts(":0");
	else
		lcd_puts(":");
	itoa(mytime.seconds, string, 10);
	lcd_puts(string);
	if(mytime.milliseconds < 100)
		if(mytime.milliseconds < 10)
			lcd_puts(";00");
		else
			lcd_puts(";0");
	else
		lcd_puts(";");
	itoa(mytime.milliseconds, string, 10);
	lcd_puts(string);
}


//Hauptprogramm
int main()
{

	init();								//initialisiert LCD-Display und Timer

	mytime.hours = 19;					//!testweise! Voreinstellung der internen Zeit
	mytime.minutes = 0;
	mytime.seconds = 16;
	mytime.milliseconds = 0;

	lcd_printTime();					//erste Ausgabe der Zeit auf dem LCD-Display

	while(1);							//leere Schleife um auf Interrupts zu warten

	return 0;
}


//Interrupts
ISR (TIMER1_COMPA_vect)					//Timer1 nach 1 s
{
	incrementTimeS();
	lcd_printTime();
}
